
%% clean up
close all
clear all
clc

%% load data
'Loading data...'
newData1 = importdata('logg_test_bil_aktiv_06_07_2010.txt');

% Create new variables in the base workspace from those fields.
for i = 1:size(newData1.colheaders, 2)
    assignin('base', genvarname(newData1.colheaders{i}), newData1.data(:,i));
end

%% fix data
'Fixing data orientation...'

% length of data
range = 1:length(seconds);

% find timestamp
time = zeros(length(range),1);
for i = 1:length(range)

    time(i) = ...
        hour(i)*60*60*1000+minute(i)*60*1000+seconds(i)*1000+nano(i)*10^-6;
    
end

% convert time to seconds
log.time = (time-min(time))/1000;

% rotation matrix from sensor frame to vehicle body frame
R = [1  0  0;
     0  1  0;
     0  0  1];   

% create measurement vectors
acc = [acc_x acc_y acc_z]';
gyr = [gyr_x gyr_y gyr_z]';
vel = [vel_x vel_y vel_z/100]';
ori = [euler_roll euler_pitch euler_yaw]';

% allocate logdata vector size
log.acc = zeros(length(range),3);
log.gyr = zeros(length(range),3);
log.vel = zeros(length(range),3);
log.ori = zeros(length(range),3);
 
for i = 1:length(range)

    % store inertial velocities
    log.vel_in(i,:) = (vel(:,i))';
   
    % store roll, pitch and yaw
    log.ori(i,:) = ori(:,i)';
    log.orgheading(i) = ori(3,i); 
    
    % find velocities in sensor frame
    R_x = [1    0                   0;
           0    cosd(log.ori(i,1)) -sind(log.ori(i,1));
           0    sind(log.ori(i,1))  cosd(log.ori(i,1))]; 
       
    R_y = [cosd(log.ori(i,2))   0   sind(log.ori(i,2));
           0                    1   0;
          -sind(log.ori(i,2))   0    cosd(log.ori(i,2))];  
       
    R_z = [cosd(log.ori(i,3))  -sind(log.ori(i,3)) 0;
           sind(log.ori(i,3))   cosd(log.ori(i,3)) 0;
           0                    0                  1];

    log.vel_s(i,:) = ( (R_z*R_y*R_x)' * log.vel_in(i,:)')';     
    
    % rotate velocity and mesaurements from sensor frame to vehicle body
    % frame
    log.vel_b(i,:) = ( R * log.vel_s(i,:)' )';       
    log.acc(i,:)   = ( R * acc(:,i) )';
    log.gyr(i,:)   = ( R * gyr(:,i) )';    
    
    % rotate yaw 180 degrees
    log.ori(i,3) = ori(3,i) + 180;
    if abs(log.ori(i,3) > 180)
       log.ori(i,3) = log.ori(i,3) - 360*sign(log.ori(i,3));
    end
end

% correct servo signal neutral point and convert to simulator
% actuator input (integer between 0-254)
log.steering = (servo1/500 - 1.54)*254 + 127;
log.throttle = (servo2/500 - 1.46)*254 + 127;

% transform longitud, latitude, altitude to NED frame
'Transform position to NED coordinates...'

% preallocate vectors
x_ned = zeros(length(range), 1);
y_ned = zeros(length(range), 1);

llh0 = [lon(1) lat(1) alt(1)];
for i=1:length(range)
   llh = [lon(i) lat(i) alt(i)];
    
   dned = dllh2dned(llh, llh0);
    
   x_ned(i) = dned(1);    
   y_ned(i) = dned(2);
    
end

log.pos = [x_ned y_ned];

%% locate test data
'Locating tests in log data...'
numTests = 2;

for test = 1:numTests
    
    % look up test time range
    switch test
        case 1
            startsAtTime = 323;
            stopsAtTime = 330;
        case 2
            startsAtTime = 480;
            stopsAtTime = 515;
        case 3
            startsAtTime = 520;
            stopsAtTime = 550;
    end
    
    % find start and stop index
    StartIndex = 0;
    EndIndex = 0;
    i = 1;
    
    while ~StartIndex
        if log.time(i) > startsAtTime
            StartIndex = i;
        end
        i = i + 1;
    end
       
    while ~EndIndex
        if log.time(i) > stopsAtTime
            EndIndex = i;
        end        
        i = i + 1;
    end
    
    testData(test).Time     = log.time(StartIndex:EndIndex);
    testData(test).Acc      = log.acc(StartIndex:EndIndex, :);
    testData(test).Gyr      = log.gyr(StartIndex:EndIndex, :);
    testData(test).Vel_b    = log.vel_b(StartIndex:EndIndex, :);
    testData(test).Vel_s    = log.vel_s(StartIndex:EndIndex, :);
    testData(test).Vel_in   = log.vel_in(StartIndex:EndIndex, :);
    testData(test).Ori      = log.ori(StartIndex:EndIndex, :);
    testData(test).orgheading = log.orgheading(StartIndex:EndIndex);
    testData(test).Pos      = log.pos(StartIndex:EndIndex, :);
    testData(test).Throttle = log.throttle(StartIndex:EndIndex);
    testData(test).Steering = log.steering(StartIndex:EndIndex);
    
end

%% plot log data
'Creating plots...'
close all

% which test to plot
test = 2;

% plot map
figure()
plot(testData(test).Pos(:,2), testData(test).Pos(:,1))
grid on
axis tight
axis equal
xlabel 'East [m]'
ylabel 'North [m]'
title ('Map of Testdrive', 'Interpreter','Latex','Fontsize', 14)

% add timestamps
for i = 1:100:length(testData(test).Pos(:,2))
    text(testData(test).Pos(i,2), testData(test).Pos(i,1), ...
        strcat('\leftarrow t=', int2str(testData(test).Time(i)), 's'))
end
set(gcf,'Position', [600   400   800   600]) 

% plot data
figure()

% plot velocity body
handle(1) = subplot(411);
hold on
plot(testData(test).Time, testData(test).Vel_b(:,1))
plot(testData(test).Time, testData(test).Vel_b(:,2) ,'r')
title ('Velocity', 'Interpreter','Latex','Fontsize', 14)
grid minor
ylabel 'Velocity [m/s]'
xlabel 'Time [s]'
legend ('V_x','V_y')
axis tight

% plot accelerations
handle(2) = subplot(412);
hold on
plot(testData(test).Time, testData(test).Acc(:,1))
plot(testData(test).Time, testData(test).Acc(:,2) ,'r')
title ('Acceleration', 'Interpreter','Latex','Fontsize', 14)
ylabel 'Acceleration [m/s^2]'
xlabel 'Time [s]'
grid minor
legend ('Acc X','Acc Y')
axis tight

% plot actuator commands
handle(3) = subplot(413);
hold on
plot([testData(test).Time(1) testData(test).Time(end)], ...
    [127 127], 'g' ,'HandleVisibility','off')
plot(testData(test).Time, testData(test).Throttle(:,1))
plot(testData(test).Time, testData(test).Steering(:,1) ,'r')
title ('Control Inputs', 'Interpreter','Latex','Fontsize', 14)
ylabel 'Input Value [0-254]'
xlabel 'Time [s]'
grid minor
legend ('Throttle', 'Steering')
axis tight

% plot heading
handle(4) = subplot(414);
plot(testData(test).Time, testData(test).Ori(:,3) ,'b')
title ('Heading', 'Interpreter','Latex','Fontsize', 14)
ylabel 'Heading [degrees]'
xlabel 'Time [s]'
grid minor
legend 'Heading'
axis ([testData(test).Time(1) testData(test).Time(end) -180 180])

linkaxes(handle, 'x');
set(gcf,'Position', [800   100   707   900]) 


